# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des methodes
class Hermes::Api::Dmo::HousingRequestsController < Hermes::Api::Dmo::ApplicationController
  hermes_tools 'main_user_id'

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $regex
  ]

  ACCEPTABLE_PARAMS = %w[
    ids
    als_id
    is_active
    request_status
    attributed_housing_district
    attributed_housing_district_postal_code
    old_request_status
    request_status_updated_at
    old_entity_status
    updated_entity_status
    entity_status
    dalo
    asked_mobility
    valid_mobility
    decider
    submitted_to_cal_nb
    sent_to_cal_nb
    rejected_in_cal_nb
    rejected_after_cal_nb
    accepted_in_cal_nb
    mobility
    seeked_types
    entity_help_required
    contract_type
    registration_number
    applicant_ids
    entity_validated
    priority_points_indexes
    nud
    nb_of_requests_made
    reference_tax_revenues
    seeked_districts
    category_type
    single_parent
    id_counter
    id_unique
    id_unique_associate
    internal_ref
    forcing_duplicate
    electronical_renewal
    ec_law_ressources
    is_long_delay_anormal
    date_calculator_indicator
    with_car_park
    widen_request
    housing_type_list
    expulsion_judgement_date
    other_reason
    work_contract_type
    seeked_district_postal_codes
    seeked_departments
    seeked_localisation_list
    dependents
    monthly_revenues
    typology_range
    entity_validated
    family_size
    coapplicants
    lessor_name
    lessor_siren
    social_housing_entry_date
    social_housing_name
    test_created_at
    sne_number
    employee_id
    employee_start_date
    entity_access_level
    employee_unit
    tab_state
    motivation1
    motivation2
    motivation3
    seeking_first_floor
    seeking_elevator
    seeked_max_rent
    sne_creation_date
    precisions
    has_multiple_attributions_under_conditions
    seeked_housing_categories
    min_surface
    main_applicant_id
    main_applicant
    requester_firstname
    requester_lastname
    requester_fullname
    requester_email
    requester_birth_date
    entity_id
    coapplicant_ids
    coapplicants
    seeked_epci_first_quartils
    seeked_quartil_belongings
    dependents_monthly_incomes
    dependents_monthly_charges
    housing_offer_ids
    radiation_date
    radiation_motive
    documents_required
    documents_uploaded
    applications_nb
    last_application_date
    is_house_owner
    disability_details
    unborn_children
    owned_housing_details
    actual_housing_details
    postal_details
    has_priority
  ]

  REGEX_PARAMS = %w[
  ]

  MUTATE_PARAMS = %i[
    workplace_insee_code
    is_active
    request_status
    attributed_housing_district
    attributed_housing_district_postal_code
    entity_status
    dalo
    asked_mobility
    valid_mobility
    decider
    mobility
    seeked_types
    entity_help_required
    contract_type
    registration_number
    applicant_ids
    entity_validated
    priority_points_indexes
    nud
    reference_tax_revenues
    seeked_districts
    id_counter
    id_unique
    id_unique_associate
    internal_ref
    forcing_duplicate
    electronical_renewal
    ec_law_ressources
    is_long_delay_anormal
    date_calculator_indicator
    with_car_park
    widen_request
    housing_type_list
    expulsion_judgement_date
    other_reason
    work_contract_type
    seeked_district_postal_codes
    seeked_departments
    seeked_localisation_list
    dependents
    entity_validated
    lessor_name
    lessor_siren
    social_housing_entry_date
    social_housing_name
    test_created_at
    sne_number
    employee_id
    employee_start_date
    entity_access_level
    employee_unit
    tab_state
    motivation1
    motivation2
    motivation3
    seeking_first_floor
    seeking_elevator
    seeked_max_rent
    sne_creation_date
    precisions
    has_multiple_attributions_under_conditions
    seeked_housing_categories
    main_applicant_id
    coapplicants
    seeked_epci_first_quartils
    seeked_quartil_belongings
    dependents_monthly_incomes
    dependents_monthly_charges
    submitted_to_cal_nb
    sent_to_cal_nb
    rejected_in_cal_nb
    rejected_after_cal_nb
    accepted_in_cal_nb
    radiation_date
    radiation_motive
    documents_required
    documents_uploaded
    is_house_owner
    disability_details
    unborn_children
    owned_housing_details
    actual_housing_details
    postal_details
  ]
  mutate_params_only(*MUTATE_PARAMS)

  def include_relationships
    [:housing_applications]
  end

  def index
    acceptable_params *ACCEPTABLE_PARAMS
    regex_params *REGEX_PARAMS
    valid_operators *VALID_OPERATORS
    super
  end

  def create
    @additional_params = { main_user_id: main_user_id_param } if main_user_id_param
    super
  end

  def update
    @additional_params = { main_user_id: main_user_id_param } if main_user_id_param
    super
  end

  def clp_details
    return render_json_error(:missing_parameter, :als_ids_param_is_missing) unless params[:als_ids].present?

    @options = { detailed_eligibility: true }
    paginate(
      index_aggregation.where(:als_id.in => (params[:als_ids] || [])),
      each_serializer: @serializer,
      **@options
    )
  end

  def housing_request_folder
    render(
        jsonapi: @instance.housing_request_folder ,
        serializer: Hermes::HousingRequestFolderSerializer,
        base_url: request.host,
        **@options
    )
  end
end
