# frozen_string_literal: true
=begin


# exemple d'utilisation
=begin

  hr = Hermes::HousingRequest.find("5e0f3f26dfbd977dd908499a")

 so=Hermes::BoClient::AddRequestTask.new({housing_request_id: hr.id})
 so.to_xml

ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml)
=end

module Hermes
  module BoClient
    class AddRequestTask < BaseTask
      
      def initialize(options = {})
        @xsd = "InterfaceNuu_v0300.xsd"
        super(options)
      end
      
      endpoint {'/SWSREC/ws/rest/als/demande/v1/integrer'}
      
      task_name 'addrequest'
      type_flux 'BIEN'
      mapping 'Hermes::BoClient::Mappings::HousingRequestMap'
      
      params *%i[
                numGuichet
                refInterne
                dateCreationDemande
                forcerDoublon
                renouvellementElec
                categorieLogementRecherche
                rezDeChaussee
                sansAscenseur
                montantLoyer
                demandeElargie
                typeLogement
                commune
                codePostal
                motifDemande
                civilite
                nom
                prenom
                dateNaissance
                nationalite
                situationFamiliale
                etranger
                batiment
                numero
                voie
                codePostal
                commune
                typeContratTravail
                multiEmployeur
                anneeMoins1
                montantMoins1
                etranger
                numero
                voie
                codePostal
                commune
                modeleLogement
                montantMensuelLoyer
                montantMensuelAPL
                nombreHabitant
                surface
                proprietaire
                etranger
            }
      ]
      
      def add_if_exists(b, item_name,mapping_name=nil, is_code = false)
        mapping_name ||=item_name
         mapping_value = mapping.send(mapping_name)
         
          b.send(item_name, is_code ? {"code" =>mapping_value} : mapping_value) unless mapping_value == nil
        
      end
      
      def to_xml
        namespaces = {
            'xmlns' => 'http://nuu.application.i2/',
            'xmlns:xsi' => "http://www.w3.org/2001/XMLSchema-instance",
            'versionInterface' => "03.00"
        }
        builder = Nokogiri::XML::Builder.new(encoding: 'UTF-8') do |b|
          b.interfaceNuu(namespaces) do
            b.parent.namespace = nil
            b.entete do
              b.typeFichier "CRE" #TODO : A modifier une fois que la gestion de doc aura ete implementée
              b.dateFichier DateTime.current
            end
            b.demande do
              b.demandeLogement do
                add_if_exists b,  "numGuichet"
                add_if_exists b,  "numUnique"
                
                add_if_exists b,  "refInterne"
                add_if_exists b,  "dateCreationDemande"
                add_if_exists b,  "forcerDoublon"
                add_if_exists b,  "renouvellementElec"
                add_if_exists b,  "precision"
                b.logementRecherche do
                  add_if_exists b,"categorieLogementRecherche","categorieLogementRecherche", true
                  add_if_exists b,  "rezDeChaussee"
                  add_if_exists b,  "sansAscenseur"
                  add_if_exists b,  "montantLoyer"
                  add_if_exists b,  "demandeElargie"
                  b.listeTypeLogement do
                    mapping.typeLogement&.each do |type|
                      b.typeLogement({"code" => type})
                    end
                  end
                  #todo : peut etre qu'il faut revoir ici ce qu'il y a a faire
                  b.listeLocalisationSouhaite do
                    mapping.localisationSouhaite&.each_with_index do |value, i|
                      b.localisationSouhaite do
                        b.commune({"code" => value[:code]})
                       ( b.codePostal value[:codePostal]) if value[:codePostal]
                      end
                    end
                  end
                end
                b.principauxMotifDemande do
                  mapping.motifDemande&.each do |motif|
                    b.motifDemande({"code"=>motif})
                  end
                  
                end
                b.personnePhysique do
                  b.demandeur do
                    add_if_exists b,"civilite","civilite", true
                    add_if_exists b,  "nom"
                    add_if_exists b,  "prenom"
                    add_if_exists b,  "dateNaissance"
                    add_if_exists b,"nationalite","nationalite", true
                    add_if_exists b,"situationFamiliale","situationFamiliale", true
                    add_if_exists b,  "mel"
                    b.adressePostale do
                      add_if_exists b,  "etranger" ,"etrangerDem"
                      add_if_exists b,  "appartement" ,"appartementDem"
                      add_if_exists b,  "etage" ,"etageDem"
                      add_if_exists b,  "escalier" ,"escalierDem"
                      add_if_exists b,  "batiment" ,"batimentDem"
                      add_if_exists b,  "compAdresse" ,"compAdresseDem"
                      add_if_exists b,  "numero" ,"numeroDem"
                      add_if_exists b,  "voie" ,"voieDem"
                      add_if_exists b,  "lieuDit" ,"lieuDitDem"
                      add_if_exists b,  "codePostal" ,"codePostalDem"
                      add_if_exists b,"commune","communeDem", true
                     # add_if_exists b,  "nomHebergeant", "nomHebergeantDem"
                    end
                    b.situationProfessionnelle do
                      add_if_exists b,"typeContratTravail","typeContratTravail", true
                      add_if_exists b,  "multiEmployeur"
                    end
                    b.revenuFiscal do
                      add_if_exists b,  "anneeMoins1"
                      add_if_exists b,  "montantMoins1"
                    end
                    b.listeRessourceRecue do
                      mapping.listeRessourceRecue&.each do |item|
                        b.detailRessource do
                          b.ressource({"code" => item & [:"code"]})
                          b.montant item & ["montant"]
                        end
                      end
                    end
                    b.listeRessourceVersee do
                      mapping.listeRessourceVersee&.each do |item|
                        b.detailRessource do
                          b.ressource({"code" => item & [:"code"]})
                          b.montant item & ["montant"]
                        end
                      end
                    end
                    b.listeRessourceRecuePersACharge do
                      mapping.listeRessourceRecuePersACharge&.each do |pers|
                        next unless pers.respond_to?(:each)
                        
                        b.listeRessourceRecue do
                          (incomes = pers&.fetch('monthly_incomes', nil)) &&
                              incomes.respond_to?(:each) &&
                              incomes&.each do |item|
                                b.detailRessource do
                                  b.ressource({"code" => item & [:"code"]})
                                  b.montant item & ["montant"]
                                end
                              end
                        end
                      end
                    end
                    b.listeRessourceVerseePersACharge do
                      mapping.listeRessourceVerseePersACharge&.each do |pers|
                        next unless pers.respond_to?(:each)
                        
                        b.listeRessourceVersee do
                          (charges = pers&.fetch('monthly_charges', nil)) &&
                              charges.respond_to?(:each) &&
                              charges&.each do |item|
                                b.detailRessource do
                                  b.ressource({"code" => item & [:"code"]})
                                  b.montant item & ["montant"]
                                end
                              end
                        end
                      end
                    end
                  end
                  b.adresseLogement do
                    add_if_exists b,  "etranger" ,"etrangerLog"
                    add_if_exists b,  "appartement" ,"appartementLog"
                    add_if_exists b,  "etage" ,"etageLog"
                    add_if_exists b,  "escalier" ,"escalierLog"
                    add_if_exists b,  "batiment" ,"batimentLog"
                    add_if_exists b,  "compAdresse" ,"compAdresseLog"
                    add_if_exists b,  "numero" ,"numeroLog"
                    add_if_exists b,  "voie" ,"voieLog"
                    add_if_exists b,  "lieuDit" ,"lieuDitLog"
                    add_if_exists b,  "codePostal" ,"codePostalLog"
                    add_if_exists b,"commune","communeLog", true
                    #add_if_exists b,  "nomHebergeant" ,"nomHebergeantLog"
                  end
                  b.logementActuel do
                    b.listeModeleLogement do
                      add_if_exists b,"modeleLogement","modeleLogement", true
                    end
                    add_if_exists b,  "montantMensuelLoyer"
                    add_if_exists b,  "montantMensuelAPL"
                    add_if_exists b,  "nombreHabitant"
                    add_if_exists b,  "surface"
                    add_if_exists b,  "proprietaire"
                    add_if_exists b,  "etranger", "etrangerLogAct"
                  end
                end
              end
            end
          end
        end
        builder.to_xml
      end
    end
  end
end
